%% tracker3D.m
%  written by Sam Daly
%       Yusuf Hamied Department of Chemistry, Lensfield Road, 
%       University of Cambridge, Cambridge, CB2 1EW, UK

%  Groups 3D coordinates from a .csv file (i.e. [x y z f]) into molecular 
%  trajectories based on user defined metrics. This code has been adapted 
%  from track.m by John C. Crocker (University of Chicago). For further 
%  details see the description within the track function. 

%  When save = 'yes', an output folder is generated in the input directory 
%  and saves results there.

% Inputs: .csv file of 3D localisations (requires x, y, z and frame).
% Outputs: 1) tracks3D.csv - all 3D tracks [x y z f molec].
%          2) Dcoeff_MLE.csv - xyz diffusion coefficients of filtered
%          tracks.
%          3) info.txt - file containing key inputs and outputs.
%          4) MLE_output.fig - histograms of diffusion coefficients.
%          5) tracks.fig - tracks plotted in 3D by diffusion coefficient.
%          6) MLE_Error.fig - 2D histogram of error vs. track length. 

% Tested in MATLAB 2022b.

clear all; close all; clc;
addpath('lib')

%% USER INPUT REQUIRED %%

% Configuration of 2D/3D localisation file
type = 'SMLFM'; % 'SMLFM' & 'DHPSF'
offset = 20; % um (change this value to ensure data does not cross y = 0 or x = 0)

save = 'no'; % 'yes' & 'no'

% Parameters 
maxdisp = 0.5; % maximum linking distance in microns
exposure = 20/1000; % camera exposure time in seconds
    param.mem = 3; % number of dark frames in frames
    param.good = 4; % minimum good track length
    param.dim = 3; % dimensionality (i.e. 2D or 3D)
    param.quiet = 0; % 1 supresses output
    
% Load data (GUI to select .csv file)
[file,path] = uigetfile('.csv', 'Select a .csv file');

input = readmatrix([path file]); input(:,1) = input(:,1) + offset; % (ensures data does not cross y = 0 or x = 0)

if type == 'DHPSF';
   positionlist = zeros(numel(input(:,1)),4);
   positionlist(:,1) = input(:,1)/1000;
   positionlist(:,2) = input(:,2)/1000; 
   positionlist(:,3) = input(:,3)/1000; 
   positionlist(:,4) = input(:,5)*exposure; 
elseif type == 'SMLFM';
   positionlist = zeros(numel(input(:,1)),4);
   positionlist(:,1) = input(:,1); 
   positionlist(:,2) = input(:,2);
   positionlist(:,3) = input(:,3); 
   positionlist(:,4) = input(:,8)*exposure; 
end

%% MAIN TRACKING CODE %%
% Calculate tracks
result = track(positionlist, maxdisp, param); % tracking function written by John C. Crocker (University of Chicago)
result(:,1) = result(:,1) - offset; % (reverts shifted data back to original position) 

% Transfer tracks to a cell array
for i = 1:max(result(:,5));
    B = result(result(:,5) == i, :);    
    tracks{i,1} = B;
    tracks{i,1}(:,5) = [];
    tracks{i,1} = circshift(tracks{i,1},1,2);
end

%% MLE Diffusion

data = result;
tracks_MLE(:,1) = data(:,4);
tracks_MLE(:,2) = data(:,1);
tracks_MLE(:,3) = data(:,2);
tracks_MLE(:,4) = data(:,3);
tracks_MLE(:,5) = data(:,5);

trackN = max(tracks_MLE(:,5));

celldata = cell(max(tracks_MLE(:,5)),1);

for i = 1:max(data(:,5))
    idx = find(tracks_MLE(:,5)==i);
    celldata{i}(:,1) = tracks_MLE(idx,1);
    celldata{i}(:,2) = tracks_MLE(idx,2);
    celldata{i}(:,3) = tracks_MLE(idx,3);
    celldata{i}(:,4) = tracks_MLE(idx,4);

    celldata{i}(:,1) = celldata{i}(:,1) - celldata{i}(1,1);
end

    D.TrackNo = zeros(max(tracks_MLE(:,5)),1);
    D.TrackDataPoints = zeros(max(tracks_MLE(:,5)),1);
    D.DxyzTrans = zeros(max(tracks_MLE(:,5)),1);
    D.DxyzTransError = zeros(max(tracks_MLE(:,5)),1);
    D.DxTrans = zeros(max(tracks_MLE(:,5)),1);
    D.DxTransError = zeros(max(tracks_MLE(:,5)),1);
    D.DyTrans = zeros(max(tracks_MLE(:,5)),1);
    D.DyTransError = zeros(max(tracks_MLE(:,5)),1);
    D.DzTrans = zeros(max(tracks_MLE(:,5)),1);
    D.DzTransError = zeros(max(tracks_MLE(:,5)),1);
    
%% Prepare arrays for rotational diffusion analysis
for i = 1:max(tracks_MLE(:,5))

    data = celldata{i};

    MLEF = MLEs; % MLE calculation of each trajectory

        
%% Calculate translational diffusion parameters using MLE        
% Get trajectory coordinates
t = data(1:end,1);
TimeStep = diff(t(1:2));
X = data(1:end,2);
Y = data(1:end,3);
Z = data(1:end,4);

[TrackDataPoints,RRTrackX,DtrueX, ... 
    SigmaX,SigmaXError,RRTrackY, ...
    DtrueY,SigmaY,SigmaYError, ...
    RRTrackZ,DtrueZ,SigmaZ, ...
    SigmaZError,MinXIndex,MinYIndex, ...
    MinZIndex,DxTrans,DxTransError, ... 
    DyTrans,DyTransError,DzTrans, ...
    DzTransError,DxyzTrans,DxyzTransError,DcorrX,DcorrY,DcorrZ,N] = ...
    MLEF.transMLE(X,Y,Z,TimeStep,length(X)-1);   

    if DxyzTrans > 0 && DxTrans > 0 && DyTrans > 0 && DzTrans > 0
    
    D.TrackNo(i,1) = i;
    D.TrackDataPoints(i,1) = TrackDataPoints;
    D.DxyzTrans(i,1) = DxyzTrans*1e12;
    D.DxyzTransError(i,1) = DxyzTransError*1e12;
    D.DxTrans(i,1) = DxTrans*1e12;
    D.DxTransError(i,1) = DxTransError*1e12;
    D.DyTrans(i,1) = DyTrans*1e12;
    D.DyTransError(i,1) = DyTransError*1e12;
    D.DzTrans(i,1) = DzTrans*1e12;
    D.DzTransError(i,1) = DzTransError*1e12;

    else 

    D.TrackNo(i,1) = NaN;
    D.TrackDataPoints(i,1) = NaN;
    D.DxyzTrans(i,1) = NaN;
    D.DxyzTransError(i,1) = NaN;
    D.DxTrans(i,1) = NaN;
    D.DxTransError(i,1) = NaN;
    D.DyTrans(i,1) = NaN;
    D.DyTransError(i,1) = NaN;
    D.DzTrans(i,1) = NaN;
    D.DzTransError(i,1) = NaN;

    end

    progress = i/max(tracks_MLE(:,5))*100;
    fprintf('MLE progress: %.1f%%\n',progress)

end

    D.TrackNo = rmmissing(D.TrackNo);
    D.TrackDataPoints = rmmissing(D.TrackDataPoints);
    D.DxyzTrans = rmmissing(D.DxyzTrans);
    D.DxyzTransError = rmmissing(D.DxyzTransError);
    D.DxTrans = rmmissing(D.DxTrans);
    D.DxTransError = rmmissing(D.DxTransError);
    D.DyTrans = rmmissing(D.DyTrans);
    D.DyTransError = rmmissing(D.DyTransError);
    D.DzTrans = rmmissing(D.DzTrans);
    D.DzTransError = rmmissing(D.DzTransError);


%% Plotting

figure('Position', [200 200 1500 600]);
t2 = tiledlayout(3,3);
title(t2,'MLE Diffusion Analysis');

    % histogram of track lengths
    nexttile
    histogram(D.TrackDataPoints,'BinWidth',2*iqr(D.TrackDataPoints)/(trackN^(1/3)));
    ylabel('Frequency');
    xlabel('Track Length (points)');
    
    % histogram of diffusion coefficient in xyz
    nexttile
    histogram(D.DxyzTrans,'BinWidth',2*iqr(D.DxyzTrans)/(trackN^(1/3)));
    ylabel('Frequency');
    xlabel('{\it D}_{xyz} (µm s^{-1})');
    
    % histogram of error in diffusion coefficient in xyz
    nexttile
    histogram(D.DxyzTransError,'BinWidth',2*iqr(D.DxyzTransError)/(trackN^(1/3)));
    ylabel('Frequency');
    xlabel('\sigma_{D_{xyz}} (µm s^{-1})');
    
    % histogram of diffusion coefficient in x
    nexttile
    histogram(D.DxTrans,'BinWidth',2*iqr(D.DxTrans)/(trackN^(1/3)));
    ylabel('Frequency');
    xlabel('{\it D}_{x} (µm s^{-1})');
    
    % histogram of error in diffusion coefficient in x
    nexttile
    histogram(D.DxTransError,'BinWidth',2*iqr(D.DxTransError)/(trackN^(1/3)));
    ylabel('Frequency');
    xlabel('\sigma_{D_{x}} (µm s^{-1})');
    
    % histogram of diffusion coefficient in y
    nexttile
    histogram(D.DyTrans,'BinWidth',2*iqr(D.DyTrans)/(trackN^(1/3)));
    ylabel('Frequency');
    xlabel('{\it D}_{y} (µm s^{-1})');
    
    % histogram of error in diffusion coefficient in y
    nexttile
    histogram(D.DyTransError,'BinWidth',2*iqr(D.DyTransError)/(trackN^(1/3)));
    ylabel('Frequency');
    xlabel('\sigma_{D_{y}} (µm s^{-1})');
    
    % histogram of diffusion coefficient in z
    nexttile
    histogram(D.DzTrans,'BinWidth',2*iqr(D.DzTrans)/(trackN^(1/3)));
    ylabel('Frequency');
    xlabel('{\it D}_{z} (µm s^{-1})');
    
    % histogram of error in diffusion coefficient in z
    nexttile
    histogram(D.DzTransError,'BinWidth',2*iqr(D.DzTransError)/(trackN^(1/3)));
    ylabel('Frequency');
    xlabel('\sigma_{D_{z}} (µm s^{-1})');

% plot tracks with colour representing diffusion coefficient 
PlotTracksMLE = tracks(D.TrackNo); % select fitted MLE tracks from all tracks
diffusionPlot(PlotTracksMLE, D.DxyzTrans, parula);
titlenum = sprintf('%d trajectories', numel(D.TrackNo));
title(titlenum);
view([45 30]);

% MLE error as a function of track length 
figure(3)
histogram2(D.TrackDataPoints,D.DxyzTransError,[25 25],'DisplayStyle','tile','ShowEmptyBins','on');
xlabel('Track Length (points)');
ylabel('\sigma_{D_{xyz}} (µm s^{-1})');
colormap("turbo");
colorbar;

% median diffusion coefficient in xyz 
fprintf('D_{MLE}: %.5f\n',median(D.DxyzTrans))

%% Write output %%
switch save
    case 'yes'

    outputFolder = [path, 'Tracking_',datestr(now,'HH-MM-SS_dd-mm-yyyy'),'\'];
    mkdir(outputFolder);

    tracks3D = result;
    tracks3D(:,4) = tracks3D(:,4)/exposure; % convert time from seconds to frame

    writematrix(tracks3D, [outputFolder '\tracks3D.csv']);
    writematrix(D.DxyzTrans, [outputFolder '\Dcoeff_MLE.csv']);

    saveas(figure(1), [outputFolder '\MLE_output.fig']);
    saveas(figure(2), [outputFolder '\tracks.fig']);
    saveas(figure(3), [outputFolder '\MLE_Error.fig']);

    % write info.txt file
    fileID = fopen([outputFolder, '\info.txt'],'w');
    fprintf(fileID,'total number of tracks = %d\n',numel(tracks));
    fprintf(fileID,'number of fitted tracks = %d\n',numel(D.TrackNo));
    fprintf(fileID,'Median track length = %d \n' ,median(D.TrackDataPoints));
    fprintf(fileID,'maxdisp = %d um\n',maxdisp);
    fprintf(fileID,'\n');
    fprintf(fileID,'Parameters\n');
    fprintf(fileID,'param.mem = %d\n',param.mem);
    fprintf(fileID,'param.good = %d\n',param.good);
    fprintf(fileID,'param.dim = %d\n',param.dim);
    fprintf(fileID,'param.quiet = %d\n',param.quiet);
    fprintf(fileID,'\n');
    fprintf(fileID,'median D_{MLE}: %.5f\n',median(D.DxyzTrans));
    fprintf(fileID,'mean D_{MLE}: %.5f\n',mean(D.DxyzTrans));
    fclose(fileID);

    case 'no'
end


